
#import <appkit/appkit.h>
#import "../NEXTVegas.h"
#import "ChipPile.h"

#define MAX_POSITIONS 12
#define BV_MAXROWS 3
#define BV_MAXCOLS 4

#define BV_GENERICPLAYERSOURCE 0
#define BV_DONTCHANGEPOSITION -1

struct NVChips {
    float x;
    float y;
	id	chipPile;
	int payOff;
	int cost;
}; 


@interface BetView:View
{
    id  delegate;
    id	tableWindow;
	
    int tag;
	int viewType;
	int numRows;
	int numCols;
    
	struct NVChips chipMatrix[BV_MAXROWS][BV_MAXCOLS];
	
	BOOL	isEnabled;
	BOOL	doSound;
	id		chipSound;
	BOOL	trackingRectWasSet;
	
	id		draggingPBoard;
	int		depositToRow;
	int		depositToCol;
}

// Class Methods...
+ initialize;
+ (NXAtom *)BetViewPBoardType;
+ (const char *const*)dragTypes;

// Creating and freeing...
- initFrame:(const NXRect *)frameRect;
- free;

// Managing Related Objects...
- delegate;
- setDelegate:delegateObject;
- chipPileAtRow:(int)r col:(int)c;


// Displaying the View
- setChipPosition:(float)x :(float)y atRow:(int)r col:(int)c;
- setChipRow:(float)y atRow:(int)row;
- setChipColumn:(float)x atCol:(int)col;
- getChipPosition:(float *)x :(float *)y atRow:(int)r col:(int)c;
- setNumRows:(int)rows;
- setNumCols:(int)cols;
- (int)numRows;
- (int)numCols;
- positionChips;
- positionRows;
- positionColumns;
- useChipSize:(int)chipSize;

// Placing Bets...
- bet:(int)units atRow:(int)r col:(int)c;
- removeBet:(int)units atRow:(int)r col:(int)c;
- switchBetOnOffAtRow:(int)r col:(int)c;

// Paying Off Bets...
- clearBets;
- clearBetsAtRow:(int)r;
- clearBetsAtColumn:(int)c;
- clearBetAtRow:(int)r col:(int)c;
- (int)collectBetAtRow:(int)r col:(int)c;
- (int)collectBets;
- getPayoff:(int *)payoff cost:(int *)cost atRow:(int)r col:(int)c;
- payBets;
- payBetAtRow:(int)r col:(int)c;
- setPayoff:(int)payoff cost:(int)cost;
- setPayoff:(int)payoff cost:(int)cost atRow:(int)r col:(int)c;

// Modifying Bets Within Betview
- addBetsFrom:otherView;
- (int)amountAtRow:(int)r col:(int)c;
- copyBetsFrom:otherView;
- getBetsFrom:otherView;
- incrementBetsBy:(int)amount;
- incrementBetAtRow:(int)r col:(int)c by:(int)amount;
- incrementBetsAtRow:(int)r by:(int)amount;
- incrementBetsAtCol:(int)c by:(int)amount;
- multiplyBetsBy:(float)amount;
- multiplyBetAtRow:(int)r col:(int)c by:(float)amount;
- multiplyBetsAtRow:(int)r by:(float)amount;
- multiplyBetsAtCol:(int)c by:(float)amount;
- sumColumnsToRow:(int)r;
- setOff:(BOOL)flag atRow:(int)r col:(int)c;
- tradeBetsWith:otherView;

// Utility Methods...
- getRow:(int *)r andCol:(int *)c underPoint:(NXPoint *)mousePoint;
- offsetRow:(int)row by:(float)value;
- setEnabled:(BOOL)flag;
- (BOOL)isEnabled;
- setSoundEnabled:(BOOL)flag;
- (BOOL)isSoundEnabled;
- setTag:(int)aTag;
- (int)tag;
- setViewType:(int)aType;
- (int)viewType;
- setTrackingRectForWindow:aWindow;
- discardTrackingRect;

// drag and drop
- (NXDragOperation)draggingEntered:sender;
- (BOOL)performDragOperation:sender;
- (BOOL)prepareForDragOperation:sender;
- concludeDragOperation:sender;
- draggedImage:(NXImage *)image beganAt:(NXPoint *)screenPoint;
- (NXDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag;
- draggedImage:(NXImage *)image endedAt:(NXPoint *)screenPoint deposited:(BOOL)flag;
- (BOOL)ignoreModifierKeysWhileDragging;

// Debugging Methods
- printToStderr;

@end

@interface Object(BetViewDelegate)

- mouseEnteredView:sender;
- mouseExitedView:sender;
- player:aPlayer willBet:(int)betType onView:aBetView;
- (BOOL)playerWillDragChipAtRow:(int)row andCol:(int)col fromView:sender;
- playerDraggedChipAtRow:(int)row andCol:(int)col fromView:sender;
- (BOOL)playerWillDropChipAtRow:(int *)row andCol:(int *)col InView:sender;
- playerDroppedChipAtRow:(int)row andCol:(int)col inView:sender;

@end
