
#import "BetView.h"
#import "Pasteboard-BetView.h"
  
static id	_ChipSound;

#define DRAGGING_POSITION_NONE -1

const char *NVDragTypes[] = {"NVBetPBoardType", NULL};
static NXAtom NVBetPBoardType;

@implementation BetView
 
+ initialize
{
	if(self == [BetView class])
	{
    	NVBetPBoardType = NXUniqueString("NVBetPBoardType");
	
		_ChipSound = [[SoundEffect allocFromZone:[self zone]] initFromSection:"chip.snd"];
	}
	
	return self;
}

+ (NXAtom *)BetViewPBoardType
{
	return &NVBetPBoardType;
}

+ (const char *const*)dragTypes
{
	return NVDragTypes;
}

- initFrame:(const NXRect *)frameRect
{
    int r, c;
 	extern const char *NVDragTypes[];

	[super initFrame:frameRect];
	
	for(c=0; c<BV_MAXCOLS; c++)
	{
		for(r=0; r<BV_MAXROWS; r++)
		{
			chipMatrix[r][c].payOff = 1;
			chipMatrix[r][c].cost = 1;
			chipMatrix[r][c].chipPile = [[ChipPile alloc] init];
			[chipMatrix[r][c].chipPile setOwnerIndex:c];
			[chipMatrix[r][c].chipPile setDragRow:r col:c];
		}
	}
	
	[self setNumRows:1];
	[self setNumCols:BV_MAXCOLS];
	[self positionChips];

	[self registerForDraggedTypes:NVDragTypes count:1];
	
	trackingRectWasSet = NO;
	
    return self;
}
 
- free
{
	int r, c;
	
	if(trackingRectWasSet)
		[tableWindow discardTrackingRect:tag];
		
	for(c=0; c<BV_MAXCOLS; c++)
	{
		for(r=0; r<BV_MAXROWS; r++)
		{
			[chipMatrix[r][c].chipPile free];
		}
	}
	
	return [super free];
}

- delegate
{
	return delegate;
}

- setDelegate:delegateObject
{
	delegate = delegateObject;
	
	return self;
}

- chipPileAtRow:(int)r col:(int)c
{
	return chipMatrix[r][c].chipPile;
}

- display
{
    return [self  displayFromOpaqueAncestor:&bounds :1 :YES];
}

- positionChips
/*
	Default method of placing chips evenly across the view.
	
*/
{
    int col,row;
    float width = NX_WIDTH(&bounds), section, center;
	NXSize chipSize;
    	
	[chipMatrix[0][0].chipPile getPileSize:&chipSize];
	
	section = (width)/numCols;
    center = section/2;
	
	for(row = 0; row < BV_MAXROWS; row++)
	{
		for(col = 0; col < BV_MAXCOLS; col++)
		{
			[self setChipPosition:(section*col)+center-chipSize.width/2.0
								 :(row*chipSize.height+2.0)
							atRow:row 
							  col:col];
		}
	}
	
	return self;
}

- positionRows
/*
	Evenly distributes rows across view, but leaves columns intact.
*/
{
	int row, col;
	NXSize chipSize;
    	
	[chipMatrix[0][0].chipPile getPileSize:&chipSize];
	
	for(row = 0; row < numRows; row++)
	{
		for(col = 0; col < numCols; col++)
		{
			[self setChipPosition:BV_DONTCHANGEPOSITION :(row*chipSize.height+2.0) 
					atRow:row col:col];
		}	
	}

	return self;
}

- positionColumns
/*
	Evenly distributes columns across view, but leaves rows intact.
*/
{
    int col,row;
    float width = NX_WIDTH(&bounds), section, center;
	NXSize chipSize;
    	
	[chipMatrix[0][0].chipPile getPileSize:&chipSize];
    	
	section = (width)/numCols;
    center = section/2;
	
	for(row = 0; row < numRows; row++)
	{
		for(col = 0; col < numCols; col++)
		{
			[self setChipPosition:(section*col)+center-chipSize.width/2
								 :BV_DONTCHANGEPOSITION 
						    atRow:row col:col];
		}
	}
	
	return self;
}



- setNumRows:(int)rows
{
	numRows = (rows <= BV_MAXROWS) ? rows : BV_MAXROWS;
	
	return self;
}

- setNumCols:(int)cols
{
	numCols = (cols <= BV_MAXCOLS) ? cols : BV_MAXCOLS;
	
	return self;
}

- (int)numRows
{
	return numRows;
}

- (int)numCols
{
	return numCols;
}

- setEnabled:(BOOL)flag
{
	isEnabled = flag;
	
	return self;
}

- (BOOL)isEnabled
{
	return isEnabled;
}

- setSoundEnabled:(BOOL)flag
{
	doSound = flag;
	
	return self;
}

- (BOOL)isSoundEnabled
{
	return doSound;
}

- setChipPosition:(float)x :(float)y atRow:(int)r col:(int)c
{
	chipMatrix[r][c].x = (x == BV_DONTCHANGEPOSITION) ? chipMatrix[r][c].x : x;
	chipMatrix[r][c].y = (y == BV_DONTCHANGEPOSITION) ? chipMatrix[r][c].y : y;
	
	[chipMatrix[r][c].chipPile setMaxWidth:bounds.size.width - chipMatrix[r][c].x 
									maxHeight:bounds.size.height - chipMatrix[r][c].y];

	return self;
}

- setChipRow:(float)y atRow:(int)row
{
	int c;
	
	for(c=0; c<BV_MAXCOLS; c++)
		[self setChipPosition:chipMatrix[row][c].x :y atRow:row col:c];
		
	return self;
}

- setChipColumn:(float)x atCol:(int)col
{
	int r;
	
	for(r=0; r<BV_MAXROWS; r++)
		[self setChipPosition:x :chipMatrix[r][col].y atRow:r col:col];
		
	return self;
}


- getChipPosition:(float *)x :(float *)y atRow:(int)r col:(int)c
{
	*x = chipMatrix[r][c].x;
	*y = chipMatrix[r][c].y;
	
	return self;
}

- offsetRow:(int)row by:(float)value
{
	int c;

	for(c = 0; c < numCols; c++)
	{
		chipMatrix[row][c].x += value;
	}
	
	return self;
}

- offsetColumn:(int)col by:(float)value
{
	int r;

	for(r = 0; r < numRows; r++)
	{
		chipMatrix[r][col].y += value;
	}
	
	return self;
}

- getRow:(int *)row andCol:(int *)col underPoint:(NXPoint *)mousePoint
/*
	returns by reference the row and column of the topmost chip under mousePoint.
	
	In order for chip to be recognized, it must have an amount.  
	
	If no chip is recognized, then this method returns nil, else it returns self.
*/
{
	NXRect chipRect;
	int r, c;
	NXSize chipSize;
	
	// Search from highest position to lowest
	for(r = numRows-1; r >= 0; r--)
	{
		for(c = numCols-1; c >= 0; c--)
		{
			if([chipMatrix[r][c].chipPile amount])
			{
				[chipMatrix[r][c].chipPile getPileSize:&chipSize];
				// get coordinates of rect that encloses chip at this position
				NX_X(&chipRect) = chipMatrix[r][c].x;
				NX_Y(&chipRect) = chipMatrix[r][c].y;
				NX_WIDTH(&chipRect) = chipSize.width;
				NX_HEIGHT(&chipRect) = chipSize.height;
				
				// if mouse is in this rect, then we've got it
				if(NXPointInRect(mousePoint, &chipRect))
				{
					*row = r;
					*col = c;
					return self;
				}
			}
		}
	}
	
	return nil;
}


- setTag:(int)aTag
{
    tag = aTag;
	
    return self;
}

- setViewType:(int)aType
{
	viewType = aType;
	
	return self;
}

- setTrackingRectForWindow:aWindow
{
    NXRect aRect;
    
	tableWindow = aWindow;

	if(trackingRectWasSet)
		[tableWindow discardTrackingRect:tag];
	
	
    [self getFrame:&aRect];
    [superview convertRect:&aRect toView:nil];
    
    [tableWindow setTrackingRect:&aRect
                     inside:NO
                      owner:self
                        tag:tag
                       left:NO
                      right:NO];

	trackingRectWasSet = YES;

	return self;
}

- discardTrackingRect
{
	if(trackingRectWasSet)
	{
		[tableWindow discardTrackingRect:tag];
		
		trackingRectWasSet = NO;
	}
	
	return self;
}

- (int)tag
{
    return tag;
}

- useChipSize:(int)chipSize
{
	int r, c;
	
	if(chipSize == NV_CHIPSIZESMALL || chipSize == NV_CHIPSIZELARGE)
	{
		for(r=0; r<BV_MAXROWS; r++)
			for(c=0; c<BV_MAXCOLS; c++)
				[chipMatrix[r][c].chipPile setChipSize:chipSize];
	}
	
	return self;
}

- (int)viewType
{
	return viewType;
}

	/*****   Pay Off Methods   *****/

- setPayoff:(int)payoff cost:(int)cost
{
	int r, c;
	
	for(r=0; r<BV_MAXROWS; r++)
		for(c=0; c<BV_MAXCOLS; c++)
			[self setPayoff:payoff cost:cost atRow:r col:c];
	
	return self;
}

- setPayoff:(int)payoff cost:(int)cost atRow:(int)r col:(int)c
{
	chipMatrix[r][c].payOff = payoff;
	chipMatrix[r][c].cost = cost;
	
	return self;
}

- getPayoff:(int *)payoff cost:(int *)cost atRow:(int)r col:(int)c
{
	*payoff = chipMatrix[r][c].payOff;
	*cost = chipMatrix[r][c].cost;

	return self;
}

- payBets
{
	int r, c;
	
	for(r=0; r<BV_MAXROWS; r++)
		for(c=0; c<BV_MAXCOLS; c++)
			[self payBetAtRow:r col:c];
	
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	return self;
}

- payBetAtRow:(int)r col:(int)c
{
	int	payOff = chipMatrix[r][c].payOff;
	int cost = chipMatrix[r][c].cost;
	
	int amountBet = [chipMatrix[r][c].chipPile amount], amountWon;
		
	if([chipMatrix[r][c].chipPile isOn])
	{
		amountWon = payOff * (int)(amountBet/cost);
		amountWon += (int)(amountBet % cost);
		amountWon += amountBet;
		
		[chipMatrix[r][c].chipPile setAmount:amountWon];
	}
	
	return self;
}



	   /***********************
	    *  Betting Methods    *
	    ***********************/

- bet:(int)units atRow:(int)r col:(int)c
{	
	[[chipMatrix[r][c].chipPile addAmount:units] setOn:YES];
	
	if(doSound)
		[_ChipSound play:1.0 pan:0.0];
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	NXPing();
	
    return self;
}

- removeBet:(int)units atRow:(int)r col:(int)c
{
	[chipMatrix[r][c].chipPile addAmount:-units];

	if(doSound)
		[_ChipSound play:1.0 pan:0.0];
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	NXPing();

    return self;
}

/**************************************
 *         Manipulating Bets          *
 **************************************/  
- (int)amountAtRow:(int)r col:(int)c
{
	return [chipMatrix[r][c].chipPile amount];
}

- clearBets
{
	int r, c;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
			[self clearBetAtRow:r col:c];
	
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];

    return self;
}

- clearBetAtRow:(int)r col:(int)c
{
	if([chipMatrix[r][c].chipPile isOn] && [chipMatrix[r][c].chipPile amount])
	{
		[chipMatrix[r][c].chipPile empty];
	}
    
	return self;
}

- clearBetsAtRow:(int)r
{
	int c;
	
	for(c=0; c<numCols; c++)
		[self clearBetAtRow:r col:c];
	
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];
	
	return self;
}

- clearBetsAtColumn:(int)c
{
	int r;
	
	for(r=0; r<numRows; r++)
		[self clearBetAtRow:r col:c];
		
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];
	
	return self;
}

- (int)collectBetAtRow:(int)r col:(int)c
{
	int amount;
	
	amount = [chipMatrix[r][c].chipPile amount];
	[chipMatrix[r][c].chipPile empty];
	
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];

	return amount;
}

- (int)collectBets
{
	int r, c, amount = 0;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
			amount += [self collectBetAtRow:r col:c];
			
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];
			
	return amount;
}

- incrementBetsBy:(int)amount
{
	int r, c;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
			[self incrementBetAtRow:r col:c by:amount];
			
	return self;
}

- incrementBetAtRow:(int)r col:(int)c by:(int)amount
{
	[chipMatrix[r][c].chipPile addAmount:amount];
	
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];

	return self;
}

- incrementBetsAtRow:(int)r by:(int)amount
{
	int c;
	
	for(c=0; c<numCols; c++)
		[self incrementBetAtRow:r col:c by:amount];

	return self;
}

- incrementBetsAtCol:(int)c by:(int)amount
{
	int r;
	
	for(r=0; r<numCols; r++)
		[self incrementBetAtRow:r col:c by:amount];

	return self;
}

- multiplyBetsBy:(float)amount
{ 
	int r, c;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
			[self multiplyBetAtRow:r col:c by:amount];
			
	return self;
}

- multiplyBetAtRow:(int)r col:(int)c by:(float)amount
{
	[chipMatrix[r][c].chipPile multiplyBy:amount];

	[self  displayFromOpaqueAncestor:&bounds :1 :YES];

	return self;
}

- multiplyBetsAtRow:(int)r by:(float)amount
{
	int c;
	
	for(c=0; c<numCols; c++)
		[self multiplyBetAtRow:r col:c by:amount];

	return self;
}

- multiplyBetsAtCol:(int)c by:(float)amount
{
	int r;
	
	for(r=0; r<numCols; r++)
		[self multiplyBetAtRow:r col:c by:amount];

	return self;
}



- copyBetsFrom:otherView
{
	int r,c;
	id otherPile;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
		{
			otherPile = [otherView chipPileAtRow:r col:c];
			[chipMatrix[r][c].chipPile setAmount:[otherPile amount]];
			[chipMatrix[r][c].chipPile setNumChips:[otherPile numChips]];
			[chipMatrix[r][c].chipPile setOn:YES];
		}
		
	return self;
}

- getBetsFrom:otherView
{
	int r,c;
	id otherPile;

	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
		{
			otherPile = [otherView chipPileAtRow:r col:c];
			[chipMatrix[r][c].chipPile setAmount:[otherPile amount]];
			[chipMatrix[r][c].chipPile setNumChips:[otherPile numChips]];
			[otherPile empty];
			[chipMatrix[r][c].chipPile setOn:YES];
		}
    
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	[otherView display];
	NXPing();

    return self;
}
 
- addBetsFrom:otherView
{
	int r,c;
	id otherPile;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
		{
			otherPile = [otherView chipPileAtRow:r col:c];
			[chipMatrix[r][c].chipPile addPile:otherPile];
			[otherPile empty];
			[chipMatrix[r][c].chipPile setOn:YES];
		}
    
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	[otherView display];
	NXPing();

    return self;
}

- setOff:(BOOL)flag atRow:(int)r col:(int)c
{
	[chipMatrix[r][c].chipPile setOn:!flag];
    
	[self  displayFromOpaqueAncestor:&bounds :1 :YES];	
	
	return self;
}

- switchBetOnOffAtRow:(int)r col:(int)c
{
	[chipMatrix[r][c].chipPile flip];

	if(doSound)
		[_ChipSound play:1.0 pan:0.0];
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	NXPing();

	return self;
}

- sumColumnsToRow:(int)r
{
	int col, row, chipTotal;
	
	for(col=0; col<numCols; col++)
	{
		chipTotal = 0;
		for(row=0; row<numRows; row++)
		{
			if(row != r)
			{
				[chipMatrix[r][col].chipPile addPile:chipMatrix[row][col].chipPile];
				[chipMatrix[row][col].chipPile empty];
			}
		}
	}
	
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];

	return self;
}

- tradeBetsWith:otherView
{
	int r,c, ourAmount, otherAmount, ourNumChips, otherNumChips;
	id otherPile;
	
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
		{
			otherPile = [otherView chipPileAtRow:r col:c];
			otherNumChips = [otherPile numChips];
			otherAmount = [otherPile amount];
			ourNumChips = [chipMatrix[r][c].chipPile numChips];
			ourAmount = [chipMatrix[r][c].chipPile amount];
			
			[[otherPile setAmount:ourAmount] setNumChips:ourNumChips];
			[[chipMatrix[r][c].chipPile setAmount:otherAmount] 
										setNumChips:otherNumChips];
		}
    
    [self  displayFromOpaqueAncestor:&bounds :1 :YES];
	[otherView display];
	NXPing();

    return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
{
	int r, c;
	for(c=0; c<numCols; c++)
	{
		for(r=0; r<numRows; r++)
		{
				NXPoint thePoint = {chipMatrix[r][c].x, chipMatrix[r][c].y};
				
				[chipMatrix[r][c].chipPile drawChipPileAt:&thePoint];
		}
	}
		
	return self;
}

- (BOOL)acceptsFirstMouse
{
    return YES;
}
 
- mouseDown:(NXEvent *)theEvent
/*
 * player has moused down in our view, indicating that they wish to bet here.
 *
 *   if no key is down -- Bet
 *   if shift is down -- remove bet
 *   if alternate key is down - lay odds
 *   if alternate+shift is down -- remove odds
 *   if control+shift -- turn off/on
 */
{
    id player = [PBoss() currentPlayer];
    NXEvent    nextEvent;
    int        oldMask;
    
    oldMask = [window addToEventMask:NX_LMOUSEDRAGGEDMASK];
    	
	if(!isEnabled) return nil;
	
	// Drag And Drop...
    if ([NXApp peekNextEvent:NX_LMOUSEDRAGGEDMASK into:&nextEvent 
            waitFor:0.25 threshold:NX_MODALRESPTHRESHOLD])
    {                                    
		int 	row, col;
		NXPoint	mousePoint, theOffset = {0, 0}, dragPoint = {0.0, 0.0};
		NXRect	chipRect;
		int 	dragAmount;
		id		dragImage, dragChipPile;
		NXSize	chipSize, pileSize;
		int 	numChips;
		
		mousePoint = theEvent->location;
		[self convertPoint:&mousePoint fromView:nil];
		
		// Don't do drag if there's nothing to drag
		if([self getRow:&row andCol:&col underPoint:&mousePoint] == nil || 
		   ![chipMatrix[row][col].chipPile amount])
			return nil; 
			
		if(![delegate playerWillDragChipAtRow:row andCol:col fromView:self])
			return nil;
		
		[chipMatrix[row][col].chipPile getPileSize:&pileSize];
		NX_X(&chipRect) = chipMatrix[row][col].x;
		NX_Y(&chipRect) = chipMatrix[row][col].y;
		NX_WIDTH(&chipRect) = pileSize.width;
		NX_HEIGHT(&chipRect) = pileSize.height;
		
		// find out if player moused down on top or side of chip stack
		//    If on side, then player is dragging all chips, if on top, then player
		//    is dragging just the top chip.
		[chipMatrix[row][col].chipPile getChipSize:&chipSize];
		dragAmount = [[PBoss() playerAt:col] selectedChip];
		numChips = 1;
		if(chipMatrix[row][col].y + pileSize.height - chipSize.height > mousePoint.y ||
			 dragAmount > [chipMatrix[row][col].chipPile amount])
		{
			dragAmount = [chipMatrix[row][col].chipPile amount];
			numChips = [chipMatrix[row][col].chipPile numChips];
			dragPoint.x = chipMatrix[row][col].x;
			dragPoint.y = chipMatrix[row][col].y;
		}
		else
		{
			dragPoint.x = chipMatrix[row][col].x + pileSize.width - chipSize.width;
			dragPoint.y = chipMatrix[row][col].y + pileSize.height - chipSize.height;
		}
		
		// create the image and the pasteboard		
		dragChipPile = [[ChipPile alloc] 
								initType:[chipMatrix[row][col].chipPile chipType]
									size:[chipMatrix[row][col].chipPile chipSize]
								   value:dragAmount];
		[dragChipPile setNumChips:numChips];
		[dragChipPile setDragRow:row col:col];
		dragImage = [dragChipPile tempDragImage];
		[dragImage setSize:&pileSize];
		
		draggingPBoard = [Pasteboard newName:"NXDragPBoard"];
		[draggingPBoard declareTypes:&NVBetPBoardType num:1 owner:self];
		[draggingPBoard writeType:NVBetPBoardType asObject:dragChipPile];
		
		[self incrementBetAtRow:row col:col by:-dragAmount];

		// drag it
		[self dragImage:dragImage 
						 at:&dragPoint 
					 offset:&theOffset
					  event:theEvent
				 pasteboard:draggingPBoard
					 source:self
				  slideBack:YES];
				  
		[delegate playerDraggedChipAtRow:row andCol:col fromView:self];
		[dragImage free];
		[dragChipPile free];
		
	}
	else
	{
		if(theEvent->flags & NX_ALTERNATEMASK && theEvent->flags & NX_CONTROLMASK &&
			theEvent->flags & NX_SHIFTMASK)
		{
			[self printToStderr];
		}
		else if(theEvent->flags & NX_ALTERNATEMASK) 
		{
			if(theEvent->flags & NX_SHIFTMASK) 
			{
				if([delegate respondsTo:@selector(player:willBet:onView:)])
					[delegate player:player willBet:NV_REMOVEOTHER onView:self];
			}
			else
			{
				if([delegate respondsTo:@selector(player:willBet:onView:)])
					[delegate player:player willBet:NV_BETOTHER onView:self];
			}
		} 
		else if(theEvent->flags & NX_CONTROLMASK) // && theEvent->flags & NX_SHIFTMASK) 
		{
			if([delegate respondsTo:@selector(player:willBet:onView:)])
				[delegate player:player willBet:NV_BETOFF onView:self];
		} 
		else if(theEvent->flags & NX_SHIFTMASK) 
		{
			if([delegate respondsTo:@selector(player:willBet:onView:)])
				[delegate player:player willBet:NV_REMOVEBET onView:self];
		} 
		else 
		{
			if([delegate respondsTo:@selector(player:willBet:onView:)])
				[delegate player:player willBet:NV_BET onView:self];
		}
	} 
    
    [window setEventMask:oldMask];
    return self;
}

- mouseEntered:(NXEvent *)theEvent
{
    if([delegate respondsTo:@selector(mouseEnteredView:)])
		[delegate mouseEnteredView:self];
	
    return self;
}

- mouseExited:(NXEvent *)theEvent
{
    if([delegate respondsTo:@selector(mouseExitedView:)])
		[delegate mouseExitedView:self];
	
	return self;
}

/*************************************************************************************
 *                               Drag And Drop                                       *
 *************************************************************************************/
 
		   /**********************************
			*       Reciever methods         *
			**********************************/
- (NXDragOperation)draggingEntered:sender
/*
 *	Sent when a player drags a chip into us.
 */
{
	Pasteboard	*pboard;
	id source = [sender draggingSource];
	id	draggedPile;
	//int row, col;
	
	[delegate mouseEnteredView:self];

	if(!isEnabled) return NX_DragOperationNone;
	
	pboard = [sender draggingPasteboard];
	[pboard readType:NVBetPBoardType asObject:&draggedPile];

	[draggedPile getDragRow:&depositToRow col:&depositToCol];

	if(([sender draggingSourceOperationMask] & NX_DragOperationGeneric) &&
		source != self &&
		[delegate playerWillDropChipAtRow:&depositToRow 
									andCol:&depositToCol InView:self])
	{
		return NX_DragOperationGeneric;
	}
	
	return NX_DragOperationNone;
}

- (BOOL)performDragOperation:sender
/*
 *	Sent when a player drops a chip in us
 */
{
	return YES;
}
 
- (BOOL)prepareForDragOperation:sender
{
	return YES;
}

- concludeDragOperation:sender
/*
 *	Sent after a player has dropped a chip in us, this is where we perform the 
 *  drag operation.
 */
{
	Pasteboard	*pboard;
	//int r, c;
	id draggedPile;

	pboard = [sender draggingPasteboard];
	[pboard readType:NVBetPBoardType asObject:&draggedPile];

	// Get the position to add this bet to.	
	//[draggedPile getDragRow:&r col:&c];
	
	// Add this bet to this position

	[chipMatrix[depositToRow][depositToCol].chipPile addPile:draggedPile];
	[draggedPile setDragRow:depositToRow col:depositToCol];

	[delegate playerDroppedChipAtRow:depositToRow andCol:depositToCol inView:self];

	[self display];
	
	return self;
}

		   /**********************************
			*          Sender methods        *
			**********************************/

- draggedImage:(NXImage *)image beganAt:(NXPoint *)screenPoint
{
	[PBoss() playSound:NV_CHIPSOUND];
	
	return self;
}

- (NXDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag
{
	if (flag)
    {
        return NX_DragOperationGeneric;
    }
    
	return NX_DragOperationNone;
}

- draggedImage:(NXImage *)image endedAt:(NXPoint *)screenPoint deposited:(BOOL)flag
{
	
	if(!flag)
	{
		int r, c;
		id draggedPile;
		
		[draggingPBoard readType:NVBetPBoardType asObject:&draggedPile];
	
		// Get the position to add this bet to.	
		[draggedPile getDragRow:&r col:&c];
		
		[chipMatrix[r][c].chipPile addPile:draggedPile];
		[self display];
	}
	
	[PBoss() playSound:NV_CHIPSOUND];
	
	return self;
}

- (BOOL)ignoreModifierKeysWhileDragging
{
	return YES;
}

- printToStderr
{
	int r, c;
	
	fprintf(stderr, "BetView #%d numRows: %d numCols: %d\n", tag, numRows, numCols);
	for(r=0; r<numRows; r++)
		for(c=0; c<numCols; c++)
		{
			fprintf(stderr, "\tChipPile at row %d col %d (%f, %f) payoff: %d cost: %d\n",
			r, c, chipMatrix[r][c].x, chipMatrix[r][c].y, chipMatrix[r][c].payOff, 
			chipMatrix[r][c].cost);
			fprintf(stderr, "\t\t");
			[chipMatrix[r][c].chipPile printToStderr];
		}
	
	return self;
}

@end
