#import "ChipPile.h"

#define NV_NUMCHIPTYPES 5
#define NV_SMALLFONTPOINT 8
#define NV_LARGEFONTPOINT 12

static id NVChipImage[2][NV_NUMCHIPTYPES];
static id NVChipFont[2];
static NXSize NVChipSize[2][NV_NUMCHIPTYPES];

@implementation ChipPile

+ initialize
{
    int i;
	char *image[NV_NUMCHIPTYPES] = {"chipPos1.eps", "chipPos2.eps", "chipPos3.eps", "chipPos4.eps", "chipPos5.eps"};

	if(self == [ChipPile class])
	{
		// Create chip images
		for(i=0; i<NV_NUMCHIPTYPES; i++) 
		{
			NVChipImage[NV_CHIPSIZESMALL][i] = 
				[[NXImage allocFromZone:[self zone]] initFromSection:image[i]];
	
			NVChipImage[NV_CHIPSIZELARGE][i] = 
				[[NXImage allocFromZone:[self zone]] initFromSection:image[i]];
	
			// Get sizes of images
			[NVChipImage[NV_CHIPSIZESMALL][i] getSize:&NVChipSize[NV_CHIPSIZESMALL][i]];
			[NVChipImage[NV_CHIPSIZELARGE][i] getSize:&NVChipSize[NV_CHIPSIZELARGE][i]];
			
			if([NVChipImage[NV_CHIPSIZESMALL][i] lockFocus])
				[NVChipImage[NV_CHIPSIZESMALL][i] unlockFocus];
			else
			{
				fprintf(stderr, "Couldn't lock focus on chip image %d\n", i);
				[NVChipImage[NV_CHIPSIZESMALL][i] free];
			}
		}
		
		// Create chip fonts
		NVChipFont[NV_CHIPSIZESMALL] = 
			[Font newFont:"Helvetica" size:NV_SMALLFONTPOINT matrix:NX_IDENTITYMATRIX];
		NVChipFont[NV_CHIPSIZELARGE] = 
			[Font newFont:"Helvetica-Bold" size:NV_LARGEFONTPOINT
										 matrix:NX_IDENTITYMATRIX];
			
	}
	
	return self;
}

- init
{
	return [self initType:NV_CHIPTYPE1 size:NV_CHIPSIZESMALL value:0];
}

- initType:(int)cpType size:(int)cpSize value:(int)cpAmount
{
	chipType = cpType;
	chipSize = cpSize;
	isOn = YES;
	
	xOffset = 0.0;
	yOffset = 2.0;
	
	maxWidth = 60.0;
	maxHeight = 60.0;
	
	showValueInsteadOfTotal = NO;
	
	[self setAmount:cpAmount];
	
	return self;
}

- copy
{
	return [self copyFromZone:[self zone]];
}

- copyFromZone:(NXZone *)zone
{
	id newChip;
	
	newChip = [[ChipPile allocFromZone:zone] initType:chipType
											  size:chipSize
											 value:amount];
											 
	[newChip setXOffset:xOffset yOffset:yOffset];
	[newChip setMaxWidth:maxWidth maxHeight:maxHeight];
	[newChip setOwnerIndex:ownerIndex];
	[newChip setDragRow:draggedFromRow col:draggedFromColumn];
	[[newChip setTag:tag] setOn:isOn];
	
	return newChip;
}

- setXOffset:(float)xoffset yOffset:(float)yoffset
{
	xOffset = xoffset;
	yOffset = yoffset;
	
	return self;
} 

- setMaxWidth:(float)maxwidth maxHeight:(float)maxheight
{
	maxWidth = maxwidth;
	maxHeight = maxheight;
	
	return self;
}

- setChipType:(int)aValue
{
	if(aValue == NV_CHIPTYPE1 || aValue == NV_CHIPTYPE2 || 
		aValue == NV_CHIPTYPE3 || aValue == NV_CHIPTYPE4 ||
		aValue == NV_CHIPTYPE5)
	{
			chipType = aValue;
	}
	
	return self;
}

- (int)chipType
{
	return chipType;
}

- setChipValue:(int)aValue
{
	chipValue = aValue;
	
	return self;
}

- (int)chipValue
{
	return chipValue;
}

- setChipSize:(int)aValue
{
	
	if(aValue == NV_CHIPSIZELARGE || aValue == NV_CHIPSIZESMALL)
	{
		chipSize = aValue;
	}
	
	return self;
}

- (int)chipSize
{
	return chipSize;
}

- setAmount:(int)aValue
{
	if(aValue > 0)
	{
		// figure out about how many chips to draw 
		if(aValue > amount)
			numChipsToDraw++;
		else if(aValue < amount)
			numChipsToDraw--;
		
		if(numChipsToDraw < 0)
			numChipsToDraw = 0;
	
		amount = aValue;
	}
	else
	{
		amount = 0;
		numChipsToDraw = 0;
	}	
	
	return self;
}

- (int)amount
{
	return amount;
}

- setNumChips:(int)numtodraw
{
	numChipsToDraw = numtodraw;

	return self;
}

- (int)numChips
{
	return numChipsToDraw;
}


- setOwnerIndex:(int)aPlayerIndex
{
	ownerIndex = aPlayerIndex;

	return self;
}

- (int)ownerIndex
{
	return ownerIndex;
}

- setTag:(int)aTag
{
	tag = aTag;
	
	return self;
}

- (int)tag
{
	return tag;
}

- setOn:(BOOL)flag
{
	isOn = flag;
	
	return self;
}

- (BOOL)isOn
{
	return isOn;
}

- showValueInsteadOfTotal:(BOOL)flag
{
	showValueInsteadOfTotal = flag;

	return self;
}

- (BOOL)showsValue
{
	return showValueInsteadOfTotal;
}

- getChipSize:(NXSize *)size
{
	size->height = NVChipSize[chipSize][chipType].height;
	size->width = NVChipSize[chipSize][chipType].width;
	
	return self;
}

- getPileSize:(NXSize *)size
{
	
	size->height = yOffset*numChipsDrawn + NVChipSize[chipSize][chipType].height;
	size->width = xOffset*numChipsDrawn + NVChipSize[chipSize][chipType].width;
	
	return self;
}

- flip
{
	isOn = !isOn;
		
	return self;
}

- addChips:(int)numChips
/*
 *  adds a chip to the pile.  chipValue should be specified...
 */
{
	amount += numChips*chipValue;
	numChipsToDraw += numChips;
	
	return self;
}

- removeChips:(int)numChips
/*
 *  removes a chip from the pile.  chipValue should be specified...
 */
{
	amount -= numChips*chipValue;
	numChipsToDraw -= numChips;
	if(amount < 0) amount = 0;
	if(numChipsToDraw < 0) numChipsToDraw = 0;
	
	return self;
}

- addAmount:(int)units
{
	return [self setAmount:amount + units];
}

- multiplyBy:(float)units
{
	return [self setAmount:amount*units];
}

- empty
{
	return [self setAmount:0];
}

- addPile:aPile
{
	amount += [aPile amount];
	
	numChipsToDraw += [aPile numChips];
	
	return self;
}

- removePile:aPile
{
	amount -= [aPile amount];
	
	numChipsToDraw -= [aPile numChips];
	if(numChipsToDraw < 0)
		numChipsToDraw = 0;
	
	return self;
}

- drawChipPileAt:(NXPoint *)aPoint
{
	if(amount > 0)
	{
		[self drawOutlineAt:aPoint];
		[self drawContentsAt:aPoint];
	}
	
	return self;
}

- drawContentsAt:(NXPoint *)aPoint
{
	float textHeight;
	NXPoint offset = {0.0, 0.0};
	char text[120];

	if(isOn)
		if(showValueInsteadOfTotal)
			sprintf(text, "%d", chipValue);
		else
			sprintf(text, "%d", amount);
	else
		sprintf(text, "OFF");
		
	[NVChipFont[chipSize] set];	

	// get offset for text
	textHeight = (chipSize == NV_CHIPSIZESMALL) ? NV_SMALLFONTPOINT 
												: NV_LARGEFONTPOINT;
	offset.x = 
		NVChipSize[chipSize][chipType].width/2 - 
		[NVChipFont[chipSize] getWidthOf:text]/2 +
		xOffset*numChipsDrawn - 1.0;
	offset.y = 
		NVChipSize[chipSize][chipType].height/2 - textHeight/2 +
		yOffset*numChipsDrawn - 1.0;
	
	PSsetgray(NX_BLACK);
	PSmoveto(aPoint->x + offset.x, aPoint->y + offset.y);
	PSshow(text);
	
	return self;
}

- drawOutlineAt:(NXPoint *)aPoint
{
	NXPoint nextPoint, zero = {0.0, 0.0};
	
	numChipsDrawn = 0;
	
	nextPoint.x = aPoint->x;
	nextPoint.y = aPoint->y;
	
	do {
		[NVChipImage[chipSize][chipType] composite:NX_SOVER toPoint:&nextPoint];
		nextPoint.y += yOffset;
		nextPoint.x += xOffset;
		zero.x += xOffset;
		zero.y += yOffset;
		numChipsDrawn++;
	} while (numChipsDrawn < numChipsToDraw && 
			zero.y+NVChipSize[chipSize][chipType].height < maxHeight &&
			zero.x+NVChipSize[chipSize][chipType].width < maxWidth);
	
	return self;
}

- drawDragPile:sender
{
	NXPoint zero = {0.0, 0.0};
	
	[self drawOutlineAt:&zero];
	[self drawContentsAt:&zero];
	
	return self;
}

- tempDragImage
{
	id dragImage;
	NXSize size;
	
	[self getPileSize:&size];
	
	dragImage = [[NXImage alloc] initSize:&size];
	[dragImage useDrawMethod:@selector(drawDragPile:) inObject:self];
	
	return dragImage;
}

- setDragRow:(int)row col:(int)col
{
	draggedFromRow = row;
	draggedFromColumn = col;
	
	return self;
}

- getDragRow:(int *)row col:(int *)col
{
	*row = draggedFromRow;
	*col = draggedFromColumn;
	
	return self;
}


- read:(NXTypedStream *)theStream
{
    [super read:theStream];
    NXReadTypes(theStream, "iii", &amount, &chipType, &chipSize);
    NXReadTypes(theStream, "iiii", &ownerIndex, &isOn, &tag, &numChipsDrawn);
    NXReadTypes(theStream, "iii", &draggedFromRow, &draggedFromColumn, &numChipsToDraw);
    return self;
}

- write:(NXTypedStream *)theStream
{
    [super write:theStream];
    NXWriteTypes(theStream, "iii", &amount, &chipType, &chipSize);
    NXWriteTypes(theStream, "iiii", &ownerIndex, &isOn, &tag, &numChipsDrawn);
    NXWriteTypes(theStream, "iii", &draggedFromRow, &draggedFromColumn, &numChipsToDraw);
	return self;
}

- printToStderr
{
	fprintf(stderr, "[%d] $%d #chips: %d type: %d size: %d on: %d (%f, %f)\n", tag, amount, numChipsToDraw, chipType, chipSize, isOn, xOffset, yOffset);
	
	return self;
}

@end