
#import "Casino.h"
#import <sys/dir.h>


void casinoBrowserKludge(DPSTimedEntry teNumber, double now, void *userData)
{
    id obj = (id)userData;
	
	[obj openModule];
}
  

@implementation Casino

- awakeFromNib
{
    moduleList = [[ModuleList allocFromZone:[pitBoss zone]] init];
	currentGameIndex = -1;  // nothing is loaded
	
	if([moduleList count] == 0)
		[openButton setEnabled:NO];
		
	cellFont = [Font newFont:"Times-BoldItalic" size:36.0];

	return self;
}

/*
 *  The following code was taken from Solitaire.app
 */
/*----------------------------------------------------------------------------
|
|    - getGameType
|
|    returns:  (id) self
|
|-----------------------------------------------------------------------------
|
|    Go find dynamically loadable NEXTVegas games in ~/Library/NEXTVegas/Games,
|    NEXTVegas.app, and /LocalLibrary/NEXTVegas/Games.  Load their descriptions
|    into the browser.
|			
\----------------------------------------------------------------------------*/
- loadBrowser:sender
{
    id theMatrix;
    char buf[MAXPATHLEN+1];
    NXSize	interCellSpacing = {0.0, 0.0}, oldSize;
    
    strcpy(buf, NXHomeDirectory());
    strcat(buf, "/Library/NEXTVegas/Games");

    if (!moduleList)
    {
        moduleList = [[ModuleList allocFromZone:[pitBoss zone]] init];
    }
    
    [self loadGamesFrom:[self appDirectory]];
	[self loadGamesFrom:buf];
    [self loadGamesFrom:GAME_DIRECTORY];

    if ([moduleList count] == 0)
    {
		NXRunAlertPanel([NXApp appName], "No games found on local filesystem!",
			"Terminate", NULL, NULL);
		[NXApp terminate:self];
    }
	else
		[openButton setEnabled:YES];
    
    [moduleList sort];
    browserValid = NO;
    [browser loadColumnZero];
    theMatrix = [browser matrixInColumn:0];

	// resize the cells in the matrix
	[theMatrix getCellSize:&oldSize];
	oldSize.height = 50.0;
	[theMatrix setCellSize:&oldSize];
	[theMatrix setIntercell:&interCellSpacing];
	[theMatrix setFont:cellFont];    
	[theMatrix sizeToCells];
	[theMatrix setAutosizeCells:YES];
	[browser display];

    return self;
}


/*----------------------------------------------------------------------------
|
|    - setGameType:(const char *)filename dir:(const char *)dirname
|
|    returns:  (id) self
|
|-----------------------------------------------------------------------------
|
|    Load the description of the specified BUNDLE_EXTENSION bundle into the 
|    browser.
|			
\----------------------------------------------------------------------------*/

- setGameType:(const char *)filename dir:(const char *)dirname
{
    id theMatrix;

    if (!moduleList)
    {
        moduleList = [[ModuleList allocFromZone:[pitBoss zone]] init];
    }

    browserValid = NO;
    [self addGameToList:filename dir:dirname];
    [moduleList sort];
    [browser loadColumnZero];
    theMatrix = [browser matrixInColumn:0];
    [theMatrix selectCellAt:realGameIndex:0];
    [theMatrix scrollCellToVisible:realGameIndex:0];

    return self;
}

/*----------------------------------------------------------------------------
|
|    - loadGamesFrom:(const char *)dirname
|
|    returns:  (id) self
|
|-----------------------------------------------------------------------------
|
|    Find all .solitaire modules in the specified directory.
|    If we find a module in several places, we keep only the first one found.
|			
\----------------------------------------------------------------------------*/

- loadGamesFrom:(const char *)dirname
{
    DIR *dir;
    struct direct *de;

    dir = opendir(dirname);
    if (dir == NULL) {
        return self;
    }

    while ((de = readdir(dir)) != NULL)
    {
    
        // Ignore '.'-files (not really necessary, I guess)
        if (de->d_name[0] == '.')
            continue;

        if (de->d_namlen > strlen(BUNDLE_EXTENSION) && 
			!strcmp(&de->d_name[de->d_namlen-strlen(BUNDLE_EXTENSION)], 
			BUNDLE_EXTENSION))
        {
            [self addGameToList:de->d_name dir:dirname];
        }
    }

    closedir(dir);
    return self;
}

/*----------------------------------------------------------------------------
|
|    - addGameToList:(const char *)filename dir:(const char *)dirname
|
|    returns:  (id) self
|
|-----------------------------------------------------------------------------
|
|    Add the specified game to the list of games.
|
\----------------------------------------------------------------------------*/

- addGameToList:(const char *)filename dir:(const char *)dirname
{
    ModuleInfo *m;
    int i, numstrings;
    char name[60];
    BOOL valid = YES;
    char path[MAXPATHLEN];
    char *iptr;
   
    numstrings = [moduleList count];
    strcpy(name, filename);

    // Smash out the '.' in "Game.BUNDLE_EXTENSION"
    if (iptr = rindex(name, '.'))
	*iptr = '\0';

    // check for duplicate modules; keep only the first one found
    for (i=0; (i < numstrings) && (valid == YES); i++)
    {
		if (!strcmp(name, [moduleList nameAt:i]))
		{
			// already have a module with this name
			valid = NO;
		}
    }
    
    if (valid) 
    {
		// add a new entry for this module
		sprintf(path, "%s/%s", dirname, filename);
		m = [[ModuleInfo allocFromZone:[pitBoss zone]]
			initWithController:NULL name:name path:path];
		if (m)
			[moduleList addObject:m];
    }
    return self;
}


- gameController
/*
*/
{
    id theController;
    
    if (![moduleList controllerAt:realGameIndex])
    {
		NXBundle *bundle;
        const char *name;
        ModuleInfo *mp;
        NXZone *theZone;
	
        mp = [moduleList objectAt:realGameIndex];
	
		// don't try to load if it failed on a previous attempt
		if ([mp failed])
		{
		    return nil;
		}
	
        name = [mp realName];
		bundle = [mp bundle];
	
		// each module gets its own zone
		theZone = NXCreateZone(vm_page_size, vm_page_size, YES);
		NXNameZone(theZone, name);

        theController = [[[bundle classNamed:name] allocFromZone:theZone] 
												      initFromBundle:bundle
														    withName:name];
		if (!theController)
		{
			[mp setFailed:YES];
			NXDestroyZone(theZone);
			NXRunAlertPanel([NXApp appName], "Loading bundle %s failed!", NULL,
	                       NULL, NULL, name);
	    	[browser setEnabled:YES];
			return nil;
		}
	
		[mp setController:theController];
    } 

    return [moduleList controllerAt:realGameIndex];
}

- moduleWasClosed
/*
	the current module was closed.  Re-enable the browser.
*/
{
	[browser setEnabled:YES];
	
	return self;
}

- openModule
/*
 *  Calls gameController to open up module represented in browser, which returns
 *  the Dealer class as the controller, and then sends that controller to pitBoss
 *  to initiate the game.
 */
{
    id myController;
	ModuleInfo *mp;

	if(kludgeTE) DPSRemoveTimedEntry(kludgeTE);
	kludgeTE = 0;


	mp = [moduleList objectAt:realGameIndex];

	if([pitBoss gameWillLoad:[mp realName]])
	{    
		// get the game controller (dynamically loads first time)
		if (myController = [self gameController])
		{
			[pitBoss newGame:myController named:[mp realName]];
			
			[[browser matrixInColumn:0] clearSelectedCell];
			
			// save selected game to defaults
			NXWriteDefault([NXApp appName], "gameType", 
							[moduleList realNameAt:realGameIndex]); 
		
			return self;
		}
		else
		{
			[pitBoss closeGameAndBringBackWelcome:YES];
		}
	}
	
	[[browser matrixInColumn:0] clearSelectedCell];
	
	return self;
}

- open:sender
/*
	Target of the browser.
	
	Since we'll swap out the welcome view for the new game view before the browser
	has a chance to unlock focus, we'll perform this little kludge to return early,
	and start a timed entry to call openModule to actually do the opening.  Otherwise,
	we'll get this nasty message on stderr:
	
	Assertion failed: You removed a View from the View hierarchy that had been lockFocus'ed

*/
{
	int index = [[browser matrixInColumn:0] selectedRow];

	[browser setEnabled:NO];
	
	if(kludgeTE)
	{
		DPSRemoveTimedEntry(kludgeTE);
		kludgeTE = 0;
	}
		
	realGameIndex = index;
	currentGameIndex = index;

	kludgeTE = DPSAddTimedEntry(.1, (DPSTimedEntryProc)casinoBrowserKludge, self, NX_BASETHRESHOLD);
	
	return self;
}

- openLastModule
/*
	Opens the last played game module.
*/
{
	[browser setEnabled:NO];
	
	[self openModule];
	
	return self;
}

- window
{
	return window;
}
 
- addCellWithString:(const char *)str at:(int)row toMatrix:matrix
{
    id theCell;
	id	theImage;
	char *iconPath = "ModuleIcon.tiff";
	
    [matrix insertRowAt:row];
    
	theCell = [matrix cellAt:row :0];
	
	if([[moduleList objectAt:row] hasIcon])
	{
		iconPath = [[moduleList objectAt:row] iconPath];
		theImage = [[NXImage allocFromZone:[self zone]] 
					initFromFile:iconPath];
	}
	else
	{
		theImage = [[NXImage allocFromZone:[self zone]]
					 initFromSection:iconPath];
	}
	
	[theCell setImage:theImage];

    [theCell setStringValue:str];
    [theCell setLoaded:YES];
    [theCell setLeaf:YES];
    return self;
}

- (int)browser:sender fillMatrix:matrix inColumn:(int)column
{
    const char *ptr;
    int i;
	
    // this shouldn't happen...
    if (browserValid) return [matrix cellCount];

    for (i = 0; i < [moduleList count]; i++)
    {
		[self addCellWithString:[moduleList nameAt: i] 
	                         at:i
		               toMatrix:matrix];
    }
           
    realGameIndex = 0;

    // select the default game
    ptr = NXGetDefaultValue([NXApp appName], "gameType");
    if (!ptr) 
		ptr = "Blackjack";
	if (ptr)
	{
		for (i = 0; i < [moduleList count]; i++)
			if (strcmp(ptr, [moduleList realNameAt:i]) == 0)
			{
				realGameIndex = i;
				break;
			}
	}
	
    browserValid = YES;

    return [matrix cellCount];
}

/*----------------------------------------------------------------------------
|
|    - (const char *)appDirectory
|
|    returns:  (const char *) full path of main bundle
|
|-----------------------------------------------------------------------------
|
|    Returns the application's main bundle directory.
|			
\----------------------------------------------------------------------------*/

- (const char *)appDirectory
{
    return [[NXBundle mainBundle] directory];
}

@end
