
#import "Player.h"

#define PLAYER_VERSION1 1
#define PLAYER_VERSION2 2
#define PLAYER_CURRENTVERSION PLAYER_VERSION2

@implementation Player

- awakeFromNib
{
    amountInSavings = 0;
	amountAtStart = 0;
     
    chip[NV_1DOLLARCHIP].view = [[chip1View setChipValue:1] setTag:NV_1DOLLARCHIP];
    chip[NV_5DOLLARCHIP].view = [[chip5View setChipValue:5] setTag:NV_5DOLLARCHIP];
    chip[NV_25DOLLARCHIP].view = [[chip25View setChipValue:25] setTag:NV_25DOLLARCHIP];
    chip[NV_50DOLLARCHIP].view = [[chip50View setChipValue:50] setTag:NV_50DOLLARCHIP];
    chip[NV_100DOLLARCHIP].view = [[chip100View setChipValue:100] setTag:NV_100DOLLARCHIP];

    chip[NV_1DOLLARCHIP].pile = [[chip1View pile] setChipType:NV_CHIPTYPE1];
    chip[NV_5DOLLARCHIP].pile = [[chip5View pile] setChipType:NV_CHIPTYPE2];
    chip[NV_25DOLLARCHIP].pile = [[chip25View  pile] setChipType:NV_CHIPTYPE3];
    chip[NV_50DOLLARCHIP].pile = [[chip50View  pile] setChipType:NV_CHIPTYPE4];
    chip[NV_100DOLLARCHIP].pile = [[chip100View  pile] setChipType:NV_CHIPTYPE5];
	
	selectedView = chip[NV_1DOLLARCHIP].view;
	 
	removeSound = [[SoundEffect allocFromZone:[self zone]] initFromSection:"removeChip.snd"];
	
    [self updateWindow];
    
    return self;
}
 
- depositToBank:(int)bankAmount
/*
 *  The only object that should use this method is the ATM.  It is used to add
 *  money to the player's bank, without tracking it with the 'wonToday' variable.
 */
{
    if(bankAmount != 0) 
	{
		if(bankAmount > 0)
			[self disperse:bankAmount];
		else
			[self remove:-bankAmount startingWith:NV_100DOLLARCHIP];
			
		amountAtStart += bankAmount;
		
		[window setDocEdited:YES];
	
		[self updateWindow];
    }
    
    return self;
}

- addToSavings:(int)savingsAmount
{
    if(savingsAmount != 0) 
	{
		amountInSavings += savingsAmount;
		
		[window setDocEdited:YES];
	
		[self updateWindow];
    }
    
    return self;
}

- (int) amountInBank
{
	int i, total=0;
	
	for(i=0; i< NV_NUMPLAYERCHIPPILES; i++)
	{
		total += [chip[i].pile amount];
	}
	
	return total;
}

- (int) amountInSavings
{
    return amountInSavings;
}

- free
{
    [window orderOut:self];
    
    return [super free];
}

- collectAllBets:sender
/*
 *  Collect all bets from table.  
 *  Not sure if this should only be able to be done when point is off or not...
 */
{
	int amount;
	
    [removeSound play:1.0 pan:0.0];
	amount = [[pitBoss dealer] collectAllBetsForPlayer:tag];
	
	if(amount > 0)
	{
		//[self addToBank:amount];
		[self disperse:amount];
		[window setDocEdited:YES];
	}
    
    [pitBoss setCurrentPlayer:self];
     
    return self;
}

- finishedBetting:sender
{
    [dealer playerDidBet:self];
    
    return self;
}

- (char *)pin
{
    return pin;
}

- (char *)playerName
{
    return playerName;
}

- (int) selectedChip
{
	return [selectedView chipValue];
}

- setPIN:(char *)pinNumber
{
    strncpy(pin, pinNumber, 4);
    [window setDocEdited:YES];
    
    return self;
}

- setPName:(const char *)name
{    
    strcpy(playerName, name);
    [window setDocEdited:YES];
	[PBoss() updatePlayerMenuAt:tag];
    
    return self;
}

- setTag:(int)aTag
{
    tag = aTag;
	
	[self updateWindow];
    
    return self;
}

- (int)tag
{
    return tag;
}

- updateWindow
{
    char buf[100];
	int i, amountInChips = [self amountInBank];
    
    sprintf(buf, "$%d", amountInChips);
    [bankText setStringValue:buf];
    
    sprintf(buf, "$%d", amountInSavings);
    [cashText setStringValue:buf];
    
    sprintf(buf, "$%d", amountInChips+amountInSavings);
    [totalText setStringValue:buf];
	
	sprintf(buf, "Amount won this session: $%d", amountInChips - amountAtStart);
	[todaysText setStringValue:buf];
	
	sprintf(buf, "[%d] %s - %s", tag+1, playerName, creationDate);
	[window setTitle:buf];
	
	for(i=0; i<NV_NUMPLAYERCHIPPILES; i++)
	{
		[chip[i].view display];
	}
    
    return self;
}

- addChip:(int)value
{
	int i, left = value, chipValue;
	
	for(i=NV_NUMPLAYERCHIPPILES-1; i>=0; i--)
	{
		chipValue = [chip[i].pile chipValue];
		while(left>0 && left >= chipValue)
		{
			[chip[i].pile addChips:1];
			//[self addToBank:chipValue];
			left -= chipValue;
		}
		[chip[i].view display];
		[self updateWindow];
		[window setDocEdited:YES];
	}
	
	
	return self;
}

- removeChip:(int)value
{
	int i, left = value, chipValue;
	BOOL okay;
	
	if([self amountInBank] < value)
	{
		NXBeep();
		return nil;
	}
	for(i=NV_NUMPLAYERCHIPPILES-1; i>=0; i--)
	{
		okay = YES;
		chipValue = [chip[i].pile chipValue];
		while(left>0 && left >= chipValue && [self amountInBank] >= chipValue)
		{
			if([chip[i].pile amount] == 0)
			{
				int j;
				
				okay = NO;
				
				// get more chips for this pile by moving up a pile and grabbing
				// a chip if it has one.  If not 
				for(j=i+1; j<NV_NUMPLAYERCHIPPILES && !okay; j++)
				{
					if([chip[j].pile amount] > 0)
					{
						[chip[j].pile removeChips:1];
						[chip[j].view display];
						[chip[i].pile addChips:[chip[j].pile chipValue]/chipValue];
						[chip[i].view display];
						okay = YES;
					}
				}
			}
			
			if(okay)
			{
				[chip[i].pile removeChips:1];
			}
			else
			{
				[self remove:chipValue startingWith:i];
			}
			//[self addToBank:-chipValue];
			left -= chipValue;
			[chip[i].view display];
			[self updateWindow];
			[window setDocEdited:YES];
		}
	}
	
	return self;
}

- disperse:(int)num
{
	return [self disperse:num startingWith:NV_100DOLLARCHIP];
}

- disperse:(int)num startingWith:(int)pileNum
/*
 *  Disperses the amount 'num' starting with the pile pileNum.  Does not affect
 *  the players bank.
 *
 *  Starts with the $100 chip, tries to fill it with as many chips as it can, then
 *  moves to the $50 chip, and so on until all the chips are dispersed.
 */
{
	int i, chipValue, total = num;
	BOOL changed = NO;
	
	for(i=pileNum; i>=0; i--)
	{
		chipValue = [chip[i].view chipValue];
		while (total > 0 && total >= chipValue)
		{
			changed = YES;
			[chip[i].pile addChips:1];
			total -= chipValue;
		}
		[chip[i].view display];
	}
	
	if(changed)
	{
		[self updateWindow];
		[window setDocEdited:YES];
	}
	
	return self;
}


- (int)remove:(int)num startingWith:(int)pileNum
/*
 *  Removes 'num' chips, but doesn't affect the player's bank.
 *
 *  returns the amount left over (0 if successful, >0 if not).
 */
{
	int i, chipValue, total = num;
	BOOL changed = NO;
	
	for(i=pileNum; i>=0; i--)
	{
		chipValue = [chip[i].view chipValue];
		while ([chip[i].pile amount] > 0 && total > 0 && total >= chipValue)
		{
			changed = YES;
			[chip[i].pile removeChips:1];
			total -= chipValue;
		}
		[chip[i].view display];
	}
	
	if(changed)
	{
		[self updateWindow];
		[window setDocEdited:YES];
	}
	
	return total;
}

- viewWasSelected:aView
/*
 *  User has single-clicked on aView.  Deselect all the other views, and make this
 *  one the selected view.
 */
{
	int i;
	
	selectedView = aView;
	
	for(i=0; i<NV_NUMPLAYERCHIPPILES; i++)
	{
		[[chip[i].view setSelected:NO] display];
	}
	[[selectedView setSelected:YES] display];
	[window setDocEdited:YES];
	
	return self;
}

- selectView:(int)viewIndex
{
	[self viewWasSelected:chip[viewIndex].view];
	
	return self;
}


- window
{
    return window;
}

- windowDidBecomeMain:sender
{
    [pitBoss setCurrentPlayer:self];
    
    return self;
}

- windowDidMove:sender
{
    [sender setDocEdited:YES];
    
    return self;
}

- windowWillClose:sender
{
    [pitBoss removePlayer:self];
    
    return nil;
}

- setSaveFile:(const char *)filename isFirstTime:(BOOL)flag
{
	if(flag)
	{
	    time_t tp;
	
		tp = time(0);
		strftime(creationDate, 1024, "%m/%d/%y", localtime(&tp));
	}
	
	if(saveFile) free(saveFile);
    
    saveFile = malloc(strlen(filename)+1);
    strcpy(saveFile, filename);
    
	[self updateWindow];
    
	playerFileVersion = PLAYER_CURRENTVERSION;
    
	return self;
}

- (char *)saveFile
{
    return saveFile;
}

- saveToTypedStream:(NXTypedStream *)typedStream
{
    char frameString[NX_MAXFRAMESTRINGLENGTH];
	int selectedChip;
	int numChips, amountInChips;
 	int i;
	
	for(i=0; i<NV_NUMPLAYERCHIPPILES; i++)
	{
		if([chip[i].view isSelected])
			selectedChip = i;
	}
	
	NXWriteType(typedStream, "i", &playerFileVersion);
	
    [window saveFrameToString:frameString];

	NXWriteTypes(typedStream, "ii", &amountInSavings, &selectedChip);
	NXWriteArray(typedStream, "c", 5, pin);
	NXWriteArray(typedStream, "c", 100, playerName);
	NXWriteArray(typedStream, "c", NX_MAXFRAMESTRINGLENGTH, frameString);
	NXWriteArray(typedStream, "c", 10, creationDate);
	
	// Fill up chips
	numChips = [chip[NV_1DOLLARCHIP].pile numChips];
	amountInChips = [chip[NV_1DOLLARCHIP].pile amount];
	NXWriteTypes(typedStream, "ii", &numChips, &amountInChips);
	
	numChips = [chip[NV_5DOLLARCHIP].pile numChips];
	amountInChips = [chip[NV_5DOLLARCHIP].pile amount];
	NXWriteTypes(typedStream, "ii", &numChips, &amountInChips);
	
	numChips = [chip[NV_25DOLLARCHIP].pile numChips];
	amountInChips = [chip[NV_25DOLLARCHIP].pile amount];
	NXWriteTypes(typedStream, "ii", &numChips, &amountInChips);
	
	numChips = [chip[NV_50DOLLARCHIP].pile numChips];
	amountInChips = [chip[NV_50DOLLARCHIP].pile amount];
	NXWriteTypes(typedStream, "ii", &numChips, &amountInChips);
	
	numChips = [chip[NV_100DOLLARCHIP].pile numChips];
	amountInChips = [chip[NV_100DOLLARCHIP].pile amount];
	NXWriteTypes(typedStream, "ii", &numChips, &amountInChips);
    
    return self;
}

- loadFromTypedStream:(NXTypedStream *)typedStream
{
    char frameString[NX_MAXFRAMESTRINGLENGTH];
	int selectedChip, temp;
	int numChips, amountInChips, amountInBank;
    
    NXReadType(typedStream, "i", &playerFileVersion);

	if(playerFileVersion == PLAYER_VERSION1)
	{	
		playerFileVersion = PLAYER_CURRENTVERSION;
		
		NXReadTypes(typedStream, "ii", &amountInBank, &amountInSavings);
		NXReadArray(typedStream, "c", 5, pin);
		NXReadArray(typedStream, "c", 100, playerName);
		NXReadArray(typedStream, "c", NX_MAXFRAMESTRINGLENGTH, frameString);
		NXReadArray(typedStream, "c", 10, creationDate);
		NXReadTypes(typedStream, "ii", &selectedChip, &temp);
		
		[window setFrameFromString:frameString];
		
		[[chip[selectedChip].view setSelected:YES] display];
		selectedView = chip[selectedChip].view;
	
		[self disperse:amountInBank];
		
		amountAtStart += amountInBank;
		
		[self updateWindow];
    }
	else
	{
		NXReadTypes(typedStream, "ii", &amountInSavings, &selectedChip);
		NXReadArray(typedStream, "c", 5, pin);
		NXReadArray(typedStream, "c", 100, playerName);
		NXReadArray(typedStream, "c", NX_MAXFRAMESTRINGLENGTH, frameString);
		NXReadArray(typedStream, "c", 10, creationDate);
		
		// Fill up chips
		NXReadTypes(typedStream, "ii", &numChips, &amountInChips);
    	[[chip[NV_1DOLLARCHIP].pile setAmount:amountInChips] setNumChips:numChips];
		NXReadTypes(typedStream, "ii", &numChips, &amountInChips);
    	[[chip[NV_5DOLLARCHIP].pile setAmount:amountInChips] setNumChips:numChips];
		NXReadTypes(typedStream, "ii", &numChips, &amountInChips);
    	[[chip[NV_25DOLLARCHIP].pile setAmount:amountInChips] setNumChips:numChips];
		NXReadTypes(typedStream, "ii", &numChips, &amountInChips);
    	[[chip[NV_50DOLLARCHIP].pile setAmount:amountInChips] setNumChips:numChips];
		NXReadTypes(typedStream, "ii", &numChips, &amountInChips);
    	[[chip[NV_100DOLLARCHIP].pile setAmount:amountInChips] setNumChips:numChips];
		
		[window setFrameFromString:frameString];
		
		[[chip[selectedChip].view setSelected:YES] display];
		selectedView = chip[selectedChip].view;
	
		amountAtStart = [self amountInBank];
		
		[self updateWindow];
	}
	
    return self;
}

@end
