// PaletteView.m
// By Jayson Adams, NeXT Developer Support Team
// You may freely copy, distribute and reuse the code in this example.
// NeXT disclaims any warranty of any kind, expressed or implied, as to its
// fitness for any particular use.

#import "PlayerView.h"
#import "../BetView.subproj/BetView.h"
#import "../BetView.subproj/Pasteboard-BetView.h"


#define PFONTSIZE 10.0

@implementation PlayerView


/* instance methods */

- initFrame:(const NXRect *)frameRect
{	
	NXSize size;
	
	isSelected = NO;
	framed = NO;
	
	chipPile = [[ChipPile alloc] init];
	
	[chipPile getChipSize:&size];
	chipPoint.x = NX_WIDTH(frameRect)/2.0 - size.width/2;
	chipPoint.y = 2.0;
	
	NVBetPBoardType = [BetView BetViewPBoardType];
	[self registerForDraggedTypes:NVBetPBoardType count:1];
	
	textFont = [Font newFont:"Helvetica-BoldOblique" size:PFONTSIZE matrix:NX_IDENTITYMATRIX];

    return [super initFrame:frameRect];
}
 
- (BOOL)acceptsFirstMouse
{
    return YES;
}

- (NXPoint) lastOffset
{
    return lastOffset;
}

- pile
{
	return chipPile;
}

- addChips:(int)num
{
	return [self addChips:num total:value];
}

- addChips:(int)num total:(int)total
{
	[[chipPile addAmount:total] setNumChips:[chipPile numChips]+num];
	
	[self display];
	
	return self;
}

- setChipValue:(int)aValue
{
	value = aValue;
	[chipPile setChipValue:value];
	
	switch(aValue)
	{
		case 1: keyEquiv = 1; break;
		case 5: keyEquiv = 2; break;
		case 25: keyEquiv = 3; break;
		case 50: keyEquiv = 4; break;
		case 100: keyEquiv = 5; break;
	}
	
	[self display];
	
	return self;
}

- setSelected:(BOOL)flag
{
	isSelected = flag;
	
	return self;
}

- (BOOL)isSelected
{
	return isSelected;
}

- (int)chipValue
{
	return value;
}

- (int)totalChipValue
{
	return [chipPile amount];
}

- setTag:(int)aTag
{
	tag = aTag;
	
	return self;
}

- (int)tag
{
	return tag;
}


- mouseDown:(NXEvent *)theEvent
{
	NXPoint	mousePoint, dragPoint, zero = {0.0, 0.0};
	NXRect	chipRect;
	id		dragImage, dragChipPile;
	NXSize	pileSize, chipSize;
	int 	numChips=1, dragAmount = value, oldMask;
	NXEvent nextEvent;
	BOOL draggingAll = NO;
    
    oldMask = [window addToEventMask:NX_LMOUSEDRAGGEDMASK];
	
	[player viewWasSelected:self];
		
    if ([NXApp peekNextEvent:NX_LMOUSEDRAGGEDMASK into:&nextEvent 
            waitFor:0.25 threshold:NX_MODALRESPTHRESHOLD])
    {                                    
		mousePoint = theEvent->location;
		[self convertPoint:&mousePoint fromView:nil];
		
		// Don't do drag if there's nothing to drag
		if([chipPile amount] == 0)
			return self;
	
		[chipPile getPileSize:&pileSize];
		NX_X(&chipRect) = chipPoint.x;
		NX_Y(&chipRect) = chipPoint.y;
		NX_WIDTH(&chipRect) = pileSize.width;
		NX_HEIGHT(&chipRect) = pileSize.height;
		
		// Make sure mouse was clicked on a pile...
		if(!NXPointInRect(&mousePoint, &chipRect))
		{
			return nil;
		}
		 
		// find out if player moused down on top or side of chip stack
		//    If on side, then player is dragging all chips, if on top, then player
		//    is dragging just the top chip.
		[chipPile getChipSize:&chipSize];
		if(NX_Y(&chipRect) + pileSize.height - chipSize.height > mousePoint.y)
		{
			draggingAll = YES;
			dragAmount = [chipPile amount];
			numChips = [chipPile numChips];
			dragPoint.x = chipPoint.x;
			dragPoint.y = chipPoint.y;
		}
		else
		{
			// need to offset the difference between bottom and top chip;
			dragPoint.x = chipPoint.x + pileSize.width - chipSize.width;
			dragPoint.y = chipPoint.y + pileSize.height - chipSize.height;
		}
		
		// create the image and the pasteboard		
		dragChipPile = [[ChipPile alloc] 
								initType:[chipPile chipType]
									size:[chipPile chipSize]
									value:dragAmount];
		[dragChipPile setNumChips:numChips];
		[dragChipPile setDragRow:BV_GENERICPLAYERSOURCE col:BV_GENERICPLAYERSOURCE];
		dragImage = [dragChipPile tempDragImage];
		[dragImage setSize:&pileSize];
		
		draggingPBoard = [Pasteboard newName:"NXDragPBoard"];
		[draggingPBoard declareTypes:NVBetPBoardType num:1 owner:self];
		[draggingPBoard writeType:*NVBetPBoardType asObject:dragChipPile];
		
		[player remove:dragAmount startingWith:tag];
		
		// drag it
		[self dragImage:dragImage 
							at:&dragPoint 
						offset:&zero
						event:theEvent
					pasteboard:draggingPBoard
						source:self
					slideBack:YES];
					
		[dragImage free];
		[dragChipPile free];
	}
 		
    [window setEventMask:oldMask];
    return self;
}


- display
{
    return [self  displayFromOpaqueAncestor:&bounds :1 :YES];
}

- drawSelf:(NXRect *)rects :(int)count
{    
	char buf[5];
	float textWidth;
	
	if(isSelected) 
	{
		PSsetgray(NX_WHITE);
		NXRectFill(&bounds);
	}
	
	[textFont set];
	PSsetgray(NX_DKGRAY);
	sprintf(buf, "$%d", value);
	PSmoveto(1.0, NX_HEIGHT(&bounds) - PFONTSIZE);
	PSshow(buf);
	
	sprintf(buf, "%d", keyEquiv);
	textWidth = [textFont getWidthOf:buf];
	PSmoveto(NX_WIDTH(&bounds) - textWidth - 1.0, NX_HEIGHT(&bounds) - PFONTSIZE);
	PSshow(buf);

	[chipPile drawChipPileAt:&chipPoint];

	PSsetgray(NX_BLACK);
	if(framed) {
		NXFrameRectWithWidth(&bounds, (NXCoord)2.0);
	}

    return self;
}

- (NXDragOperation)draggingEntered:sender
/*
 *	Sent when a player drags a chip into us.
 */
{
	id source = [sender draggingSource];
	
	if(([sender draggingSourceOperationMask] & NX_DragOperationGeneric) &&
		source != self)
	{
		framed = YES;
		[self display];
		return NX_DragOperationGeneric;
	}
	
	return NX_DragOperationNone;
}

- draggingExited:sender
{
	framed = NO;
	
	[self display];
	
	return self;
}


- (BOOL)performDragOperation:sender
/*
 *	Sent when a player drops a chip in us
 */
{
	return YES;
}

- (BOOL)prepareForDragOperation:sender
{
	return YES;
}

- concludeDragOperation:sender
/*
 *	Sent after a player has dropped a chip in us, this is where we perform the 
 *  drag operation.
 */
{
	Pasteboard	*pboard;
	id draggedPile;
	
	pboard = [sender draggingPasteboard];
	[pboard readType:*NVBetPBoardType asObject:&draggedPile];
	
	[player disperse:[draggedPile amount] startingWith:tag];

	framed = NO;
	[player viewWasSelected:self];
		
	return self;
}

- draggedImage:(NXImage *)image beganAt:(NXPoint *)screenPoint
{
	[PBoss() playSound:NV_CHIPSOUND];
	
	return self;
}

- (NXDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag
{
	if (flag)
    {
        return NX_DragOperationGeneric;
    }
    
	return NX_DragOperationNone;
}

- draggedImage:(NXImage *)image endedAt:(NXPoint *)screenPoint deposited:(BOOL)flag
{
	
	if(!flag)
	{
		id draggedPile;
		
		[draggingPBoard readType:*NVBetPBoardType asObject:&draggedPile];
	
		[player disperse:[draggedPile amount] startingWith:tag];
	}
	
	framed = NO;
	[PBoss() playSound:NV_CHIPSOUND];
	[self display];
	
	return self;
}

- (BOOL)ignoreModifierKeysWhileDragging
{
	return YES;
}

@end
