
#import "PrefController.h"

@implementation PrefController

- awakeFromNib
{
	prefMatrix = [[prefButton target] itemList];
	
	NVPrefs = [prefBox contentView];
	
	NVPrefsLoaded = YES;
	
	if([Dealer() inspector:self])
		[self allowModuleInspector:YES];
	else
		[self allowModuleInspector:NO];
		
	[self revert:self];
	
	return self;
}

- allowModuleInspector:(BOOL)hasModuleInspector
{
	if(!hasModuleInspector)
	{
		[prefMatrix selectCellWithTag:NXVEGASPREF];
		[prefButton setTitle:[[prefMatrix selectedCell] title]];
		[self setPrefToView:NVPrefs];
	}
	else
	{
		[prefMatrix selectCellWithTag:MODULEPREF];
		[prefButton setTitle:[[prefMatrix selectedCell] title]];
		[self setPrefToView:[[pitBoss dealer] inspector:self]];
	}

	[[prefMatrix findCellWithTag:MODULEPREF] setEnabled:hasModuleInspector];
	
	
	return self;
}

- prefsChanged:sender
{
	[setButton setEnabled:YES];
	[revertButton setEnabled:YES];
	[prefWindow setDocEdited:YES];
	
	return self;
}

- revert:sender
{
	BOOL wasOkay = NO;
	
	if(NVPrefsLoaded)
	{
		[houseBetsForm 
			setStringValue:NXGetDefaultValue([NXApp appName], "houseMin") at:0];
		[houseBetsForm 
			setStringValue:NXGetDefaultValue([NXApp appName], "houseMax") at:1];
		[soundEnabledButton 
			setState:atoi(NXGetDefaultValue([NXApp appName], "soundOn"))];
		[defPlayerDirText 
			setStringValue:NXGetDefaultValue([NXApp appName], "PlayerDir")];
		[openLastGameButton 
			setState:atoi(NXGetDefaultValue([NXApp appName], "openLast"))];
		[openLastPlayerButton 
			setState:atoi(NXGetDefaultValue([NXApp appName], "openLastPlayer"))];
			
		wasOkay = YES;
	}
	else
	{
		if([Dealer() revertPreferences:self])
			wasOkay = YES;
	}
	
	if(wasOkay)
	{
		[setButton setEnabled:NO];
		[revertButton setEnabled:NO];
		[prefWindow setDocEdited:NO];
	}
	
	return self;
}

- set:sender
{
    BOOL wasOkay = NO;
	
	if(NVPrefsLoaded)
	{
		static NXDefaultsVector newDefaults = {
			{"houseMin", ""},
			{"houseMax", ""},
			{"soundOn", ""},
			{"PlayerDir", ""},
			{"gameType", ""},
			{"openLast", ""},
			{"openLastPlayer", ""},
			{"lastPlayer", ""},
			{NULL, NULL}
		};
		
		// houseMin, houseMax
		newDefaults[0].value = alloca(256);
		newDefaults[1].value = alloca(256);
		newDefaults[0].value = (char *)[houseBetsForm stringValueAt:0];
		newDefaults[1].value = (char *)[houseBetsForm stringValueAt:1];
		[pitBoss setHouseMin:[houseBetsForm intValueAt:0]];
		[pitBoss setHouseMax:[houseBetsForm intValueAt:1]];
		
		// soundOn
		newDefaults[2].value = alloca(256);
		sprintf(newDefaults[2].value, "%d", [soundEnabledButton state]);
		[pitBoss setSoundEnabled:(BOOL)[soundEnabledButton state]];
		
		newDefaults[3].value = alloca(256);
		sprintf(newDefaults[3].value, "%s", [defPlayerDirText stringValue]);
		
		newDefaults[4].value = alloca(256);
		newDefaults[4].value = (char *)NXGetDefaultValue([NXApp appName], "gameType");
		
		newDefaults[5].value = alloca(256);
		sprintf(newDefaults[5].value, "%d", [openLastGameButton state]);
	
		newDefaults[6].value = alloca(256);
		sprintf(newDefaults[6].value, "%d", [openLastPlayerButton state]);
	
		newDefaults[7].value = alloca(256);
		strcpy(newDefaults[7].value, NXGetDefaultValue([NXApp appName], "lastPlayer"));
	
		NXWriteDefaults([NXApp appName], newDefaults);
		
		wasOkay = YES;
	}
	else
	{
		if([Dealer() setPreferences:self])
			wasOkay = YES;
	}
	
	if(wasOkay)
	{
		[setButton setEnabled:NO];
		[revertButton setEnabled:NO];
		[prefWindow setDocEdited:NO];
    }
	
	return self;
}

- setPref:sender
{
    id newView = nil;
	
	switch([[sender selectedCell] tag])
	{
		case NXVEGASPREF:
			newView = NVPrefs;
			break;
		case MODULEPREF:
			newView = [Dealer() inspector:self];
			break;
	}
	
	[self setPrefToView:newView];
	
	return self;
}

- setPrefToView:aView
{
    NXRect boxRect, viewRect;
	
	[prefBox getFrame:&boxRect];
	[aView getFrame:&viewRect];
	
	[prefBox setContentView:aView];
	
	NX_X(&viewRect) = (NX_WIDTH(&boxRect)-NX_WIDTH(&viewRect))/2.0;
	NX_Y(&viewRect) = (NX_HEIGHT(&boxRect)-NX_HEIGHT(&viewRect))/2.0;
	
	[aView setFrame:&viewRect];
	
	if(aView == NVPrefs)
	{
		NVPrefsLoaded = YES;
		[self revert:self];
	}
	else
	{
		NVPrefsLoaded = NO;
		[Dealer() revertPreferences:self];
	}

	[prefBox display];
	
	return self;
}

- window
{
	return prefWindow;
}

@end
